==> wayidle-1.0.0.tar.gz: wayidle-v1.0.0/src/main.rs <==
use anyhow::Context;
use lexopt::{
    prelude::{Long, Short},
    Arg::Value,
    ValueExt,
};
use std::{collections::HashMap, ffi::CString, os::unix::process::CommandExt};
use std::{os::unix::ffi::OsStrExt, process::Command};
use wayrs_client::{protocol::WlSeat, Connection, EventCtx, IoMode};
use wayrs_protocols::ext_idle_notify_v1::{
    ext_idle_notification_v1, ExtIdleNotificationV1, ExtIdleNotifierV1,
};
use wayrs_utils::seats::{SeatHandler, Seats};

fn usage() {
    println!("Usage: wayidle [-s|--seat=SEAT_NAME] -t|--timeout=SECONDS [COMMAND [ARGS...]]");
}

fn main() -> anyhow::Result<()> {
    let mut parser = lexopt::Parser::from_env();

    let mut seat_name = None;
    let mut timeout = None;
    let mut command = None;

    while let Some(arg) = parser.next()? {
        match arg {
            Short('s') | Long("seat") => {
                let c_name = CString::new(parser.value()?.as_bytes())
                    .context("seat name must not contain null bytes")?;
                seat_name = Some(c_name);
            }
            Short('t') | Long("timeout") => {
                timeout = Some(parser.value()?.parse()?);
            }
            Short('h') | Long("help") => {
                usage();
                std::process::exit(0);
            }
            Short(_) | Long(_) => return Err(arg.unexpected().into()),
            Value(prog) => {
                let mut cmd = Command::new(prog);
                cmd.args(parser.raw_args()?);
                command = Some(cmd);
            }
        }
    }
    let Some(timeout) = timeout else {
        usage();
        anyhow::bail!("No timeout specified");
    };

    wait_for_timeout(timeout, seat_name)?;

    if let Some(mut cmd) = command {
        anyhow::bail!(cmd.exec());
    }

    Ok(())
}

fn wait_for_timeout(timeout: u32, seat_name: Option<CString>) -> anyhow::Result<()> {
    let mut conn = Connection::connect()?;
    conn.blocking_roundtrip()?;

    let mut state = State {
        exit: false,
        seats: Seats::bind(&mut conn),
        seat_names: HashMap::default(),
    };
    conn.blocking_roundtrip()?; // Receive seat names.
    conn.dispatch_events(&mut state);

    let seat = match seat_name {
        Some(name) => state
            .seat_names
            .get(&name)
            .copied()
            .with_context(|| format!("seat '{}' not found", name.to_string_lossy()))?,
        None => state.seats.iter().next().context("no seats found")?,
    };
    let idle_notifier = conn.bind_singleton::<ExtIdleNotifierV1>(1..=1)?;
    idle_notifier.get_idle_notification_with_cb(&mut conn, timeout * 1000, seat, notification_cb);

    while !state.exit {
        conn.flush(IoMode::Blocking)?;
        conn.recv_events(IoMode::Blocking)?;
        conn.dispatch_events(&mut state);
    }

    Ok(())
}

struct State {
    exit: bool,
    seats: Seats,
    seat_names: HashMap<CString, WlSeat>,
}

impl SeatHandler for State {
    fn get_seats(&mut self) -> &mut Seats {
        &mut self.seats
    }

    fn seat_name(&mut self, _: &mut Connection<Self>, wl_seat: WlSeat, name: CString) {
        self.seat_names.insert(name, wl_seat);
    }
}

#[allow(clippy::needless_pass_by_value)] // Required for callback API
fn notification_cb(ctx: EventCtx<State, ExtIdleNotificationV1>) {
    if let ext_idle_notification_v1::Event::Idled = ctx.event {
        ctx.state.exit = true;
    }
}

==> wayidle-1.0.1.tar.gz: wayidle-v1.0.1/src/main.rs <==
use anyhow::Context;
use lexopt::{
    prelude::{Long, Short},
    Arg::Value,
    ValueExt,
};
use std::{collections::HashMap, ffi::CString, os::unix::process::CommandExt};
use std::{os::unix::ffi::OsStrExt, process::Command};
use wayrs_client::{protocol::WlSeat, Connection, EventCtx, IoMode};
use wayrs_protocols::ext_idle_notify_v1::{
    ext_idle_notification_v1, ExtIdleNotificationV1, ExtIdleNotifierV1,
};
use wayrs_utils::seats::{SeatHandler, Seats};

fn usage() {
    println!("Usage: wayidle [-s|--seat=SEAT_NAME] -t|--timeout=SECONDS [COMMAND [ARGS...]]");
}

fn main() -> anyhow::Result<()> {
    let mut parser = lexopt::Parser::from_env();

    let mut seat_name = None;
    let mut timeout = None;
    let mut command = None;

    while let Some(arg) = parser.next()? {
        match arg {
            Short('s') | Long("seat") => {
                let c_name = CString::new(parser.value()?.as_bytes())
                    .context("seat name must not contain null bytes")?;
                seat_name = Some(c_name);
            }
            Short('t') | Long("timeout") => {
                timeout = Some(parser.value()?.parse()?);
            }
            Short('h') | Long("help") => {
                usage();
                std::process::exit(0);
            }
            Short(_) | Long(_) => return Err(arg.unexpected().into()),
            Value(prog) => {
                let mut cmd = Command::new(prog);
                cmd.args(parser.raw_args()?);
                command = Some(cmd);
            }
        }
    }
    let Some(timeout) = timeout else {
        usage();
        anyhow::bail!("No timeout specified");
    };

    wait_for_timeout(timeout, seat_name)?;

    if let Some(mut cmd) = command {
        anyhow::bail!(cmd.exec());
    }

    Ok(())
}

fn wait_for_timeout(timeout: u32, seat_name: Option<CString>) -> anyhow::Result<()> {
    let mut conn = Connection::connect()?;

    let mut state = State {
        exit: false,
        seats: Seats::bind(&mut conn),
        seat_names: HashMap::default(),
    };

    conn.blocking_roundtrip()?; // Receive seats.
    conn.dispatch_events(&mut state);
    conn.blocking_roundtrip()?; // Receive seat names.
    conn.dispatch_events(&mut state);

    let seat = match seat_name {
        Some(name) => state
            .seat_names
            .get(&name)
            .copied()
            .with_context(|| format!("seat '{}' not found", name.to_string_lossy()))?,
        None => state.seats.iter().next().context("no seats found")?,
    };
    let idle_notifier = conn.bind_singleton::<ExtIdleNotifierV1>(1..=1)?;
    idle_notifier.get_idle_notification_with_cb(&mut conn, timeout * 1000, seat, notification_cb);

    while !state.exit {
        conn.flush(IoMode::Blocking)?;
        conn.recv_events(IoMode::Blocking)?;
        conn.dispatch_events(&mut state);
    }

    Ok(())
}

struct State {
    exit: bool,
    seats: Seats,
    seat_names: HashMap<CString, WlSeat>,
}

impl SeatHandler for State {
    fn get_seats(&mut self) -> &mut Seats {
        &mut self.seats
    }

    fn seat_name(&mut self, _: &mut Connection<Self>, wl_seat: WlSeat, name: CString) {
        self.seat_names.insert(name, wl_seat);
    }
}

fn notification_cb(ctx: EventCtx<State, ExtIdleNotificationV1>) {
    if let ext_idle_notification_v1::Event::Idled = ctx.event {
        ctx.state.exit = true;
    }
}
